/* ListInterface
 *
 * Version 2, some comments added by Nancy Griffeth
 *
 * 2001
 *
 * Copyright 2001 Addison-Wesley
 */
/**
An ordered collection (also known as a sequence). 
The user of this interface has precise control over where in the list 
each element is inserted. The user can access elements by their integer 
index (position in the list), and search for elements in the list.

Unlike sets, lists typically allow duplicate elements. More formally, 
lists typically allow pairs of elements <code>e1</code> and <code>e2</code> 
such that <code>e1.equals(e2)</code>, 
and they typically allow multiple <code>null</code> elements 
if they allow <code>null</code> elements 
at all. It is not inconceivable that someone might wish to implement 
a list that prohibits duplicates, by throwing runtime exceptions 
when the user attempts to insert them, but we expect this usage to be rare.
**/
public interface ListInterface {
/**
Returns <code>true</code> if this list contains no elements.
<p>
<strong>Pre-condition:</strong> None
<br>
<strong>Post-condition:</strong> Returns <code>true</code> if the list is empty
and <code>false</code> otherwise
<P>
@return <code>true</code> if this list contains no elements
**/
    public boolean isEmpty();
/**
Returns the number of elements in this list.
<p>
<strong>Pre-condition:</strong> None
<br>
<strong>Post-condition:</strong> Returns the number of elements in the list, or
<code>Integer.MAX_VALUE</code> if there are more than <code>Integer.MAX_VALUE</code>
elements in the list.
<p>
@return the number of elements in this list.
**/
    public int size();
/**
Inserts the specified element at the specified position in this list. 
Shifts the element currently at that position 
(if any) and any subsequent elements to the right (adds one to their indices).
<p>
<strong>Pre-condition:</strong> <code>index</code> is between <code>1</code>
and <code>this.size()+1</code>.
<br>
<strong>Post-condition:</strong> <code>item</code> becomes the list element
at position <code>index</code>; the position of later elements in the list 
is increased by <code>1</code> to make room for it. 
<p>
@param index index at which the specified element is to be inserted.
@param item element to be inserted.
<p>
@throws ListIndexOutOfBoundsException if the index is out of the range 1 up to the size of the list
@throws ListException if the item cannot be added to the list for any reason
**/
    public void add(int index, Object item) 
                  throws ListIndexOutOfBoundsException,
                         ListException;
/**
Returns the element at the specified position in this list.
<p>
<strong>Pre-condition:</strong> <code>index</code> is between <code>1</code>
and <code>this.size()</code>.
<br>
<strong>Post-condition:</strong> Returns the element at position <code>index</code>. 
<p>
@param index index of element to return.
<p>
@return the element at the specified position in this list
<p>
@throws ListIndexOutOfBoundsException if the index is out of the range 1 up to the size of the list
**/
    public Object get(int index) 
                    throws ListIndexOutOfBoundsException;
/**
Removes the element at the specified position in this list. Shifts any subsequent 
elements to the left (subtracts one from their indices). 
<p>
<strong>Pre-condition:</strong> <code>index</code> is between <code>1</code>
and <code>this.size()+1</code>.
<br>
<strong>Post-condition:</strong> Removes the element at position <code>index</code>; 
the position of later elements in the list is decreased by <code>1</code>.
<p>
@param index index of element to remove.
<p>
@throws ListIndexOutOfBoundsException if the index is out of the range 1 up to the size of the list
**/
    public void remove(int index) 
                     throws ListIndexOutOfBoundsException;
/**
Removes from this list all the elements that are contained in the 
specified collection.
<p>
<strong>Pre-condition:</strong> None.
<br>
<strong>Post-condition:</strong> The list is empty (<code>this.isEmpty()</code>
returns true).
**/
    public void removeAll();
}  // end ListInterface
